using System;
using System.Data;
using System.Configuration;
using Vevo.Shared.DataAccess;

namespace Vevo.DataAccessLib.Cart
{
    /// <summary>
    /// Summary description for OrderNoteAccess
    /// </summary>
    public static class OrderNoteAccess
    {
        private static string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy;
        }


        public static string Create(
            string orderID, string author, string note )
        {
            string noteID = DataAccess.ExecuteScalar(
                "INSERT INTO OrderNote (OrderID, Author, OrderNote) " +
                "VALUES (@OrderID, @Author, @OrderNote); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( orderID ),
                DataAccess.CreateParameterString( author ),
                DataAccess.CreateParameterString( note ) );
            return noteID;
        }

        public static DataTable GetByOrderID( string orderID, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT NoteID, OrderID, CreatedDate, Author, OrderNote " +
                "FROM OrderNote " +
                "WHERE OrderID = @OrderID " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( orderID ) );
            DataColumn[] keys = new DataColumn[1] { table.Columns["NoteID"] };
            table.PrimaryKey = keys;
            return table;
        }

    }
}
