﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Blogs;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using System.Data;
using System.Collections;
using Vevo.Domain;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class BlogCommentRepositoryNoCache : IBlogCommentRepository
    {
        #region Private
        private const string AllBlogCommentFields =
                    " BlogCommentID, BlogID, UserName, Comment, CreatedDate, IsEnabled ";

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private BlogComment Create( BlogComment blogComment )
        {
            string newID = DataAccess.ExecuteScalar(
            "INSERT INTO BlogComment( BlogID, UserName, Comment, CreatedDate, IsEnabled) " +
            "VALUES ( @BlogID, @UserName, @Comment, @CreatedDate, @IsEnabled); " +
            "SELECT @@Identity; ",
            DataAccess.CreateParameterString( blogComment.BlogID ),
            DataAccess.CreateParameterString( blogComment.UserName ),
            DataAccess.CreateParameterString( blogComment.Comment ),
            DataAccess.CreateParameterDateTime( blogComment.CreatedDate ),
            DataAccess.CreateParameterBool( blogComment.Enabled ) );

            blogComment.BlogCommentID = newID;
            return blogComment;
        }

        private BlogComment Update( BlogComment blogComment )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE BlogComment " +
                "SET " +
                "   BlogID = @BlogID, " +
                "   UserName = @UserName, " +
                "   Comment = @Comment, " +
                "   CreatedDate = @CreatedDate, " +
                "   IsEnabled = @IsEnabled " +
                 " WHERE BlogCommentID = @BlogCommentID; ",
                  DataAccess.CreateParameterString( blogComment.BlogID ),
            DataAccess.CreateParameterString( blogComment.UserName ),
            DataAccess.CreateParameterString( blogComment.Comment ),
            DataAccess.CreateParameterDateTime( blogComment.CreatedDate ),
            DataAccess.CreateParameterBool( blogComment.Enabled ),
            DataAccess.CreateParameterString( blogComment.BlogCommentID ) );

            return blogComment;
        }

        private BlogComment CreateOneFromRow( DataRow row )
        {
            BlogComment blogComment = new BlogComment();
            blogComment.BlogCommentID = row["BlogCommentID"].ToString();
            blogComment.BlogID = row["BlogID"].ToString();
            blogComment.UserName = row["UserName"].ToString();
            blogComment.Comment = row["Comment"].ToString();
            blogComment.CreatedDate = ConvertUtilities.ToDateTime( row["CreatedDate"] );
            blogComment.Enabled = ConvertUtilities.ToBoolean( row["IsEnabled"] );

            return blogComment;
        }

        private IList<BlogComment> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<BlogComment> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<BlogComment> list = new List<BlogComment>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        #endregion

        #region Public

        public BlogComment Save( BlogComment blogComment )
        {
            BlogComment result;

            if (blogComment.BlogCommentID == "0")
                result = Create( blogComment );
            else
                result = Update( blogComment );

            return result;
        }

        public BlogComment GetOne( string blogCommentID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllBlogCommentFields +
                "FROM BlogComment " +
                "WHERE BlogCommentID = @BlogCommentID; ",
                DataAccess.CreateParameterString( blogCommentID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return BlogComment.Null;
        }

        public void Delete( string blogCommentID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM BlogComment " +
                "WHERE BlogCommentID = @BlogCommentID; ",
                DataAccess.CreateParameterString( blogCommentID ) );
        }

        public IList<BlogComment> GetAllByBlogID( string blogID, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllBlogCommentFields +
                "FROM BlogComment " +
                "WHERE BlogID = @BlogID " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( blogID ) );

            return ConvertTableToList( table );
        }

        public IList<BlogComment>  SearchBlogComment(
            string blogID,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems,
            BoolFilter isEnabled )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string command = searchFilterHelper.CreateCommandWithWhere(searchFilter,searchFilter.FieldName,parameterList);
            string whereBlogID = "BlogID = " + blogID + " ";
            if (String.IsNullOrEmpty( command.Trim() ))
                whereBlogID = " WHERE " + whereBlogID;
            else
                whereBlogID = " AND " + whereBlogID;
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllBlogCommentFields +
                "FROM BlogComment " + command + whereBlogID + 
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) + 
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            howManyItems = table.Rows.Count;

            DataColumn[] keys = new DataColumn[] { table.Columns["BlogCommentID"] };
            table.PrimaryKey = keys;

            return ConvertTableToList( table, startIndex, endIndex );
        }

        public IList<BlogComment> SearchBlogComment(
            string blogID,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems,
            BoolFilter isEnabled )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string whereBlogID = "BlogID = " + blogID + " ";
            whereBlogID = " WHERE " + whereBlogID;
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllBlogCommentFields +
                "FROM BlogComment " + whereBlogID +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            howManyItems = table.Rows.Count;

            DataColumn[] keys = new DataColumn[] { table.Columns["BlogCommentID"] };
            table.PrimaryKey = keys;

            return ConvertTableToList( table, startIndex, endIndex );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                 "SELECT " + AllBlogCommentFields +
                 "FROM BlogComment " +
                 "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        #endregion
    }
}
