using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Users;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class CountryRepository : ICountryRepository
    {
        #region Private

        private const string RepositoryName = "CountryRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetOneByNameSignature = "GetOneByName";
        private const string GetAllSignature = "GetAll";
        private const string TableName = "Country";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( TableName );

        private CountryRepositoryNoCache _repository = new CountryRepositoryNoCache();

        #endregion

        #region Constructors

        public CountryRepository()
        {
        }

        public CountryRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public Country Create( Country country )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Country newCountry = _repository.Create( country );
            _dataCache.ClearRepositoryCache();
            return newCountry;
        }

        public Country Update( Country country )
        {
            Country result = _repository.Update( country );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public Country Update( Country country, string newCountryCode )
        {
            Country result = _repository.Update( country, newCountryCode );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public Country GetOne( string countryCode )
        {
            Country result = (Country) _dataCache.GetValue( GetOneSignature, _cacheDependency, countryCode );

            if (result == null)
            {
                result = _repository.GetOne( countryCode );
                _dataCache.SetValue( GetOneSignature, result, countryCode );
            }

            return result;
        }

        public Country GetOneByName( string countryName )
        {
            Country result = (Country) _dataCache.GetValue( GetOneByNameSignature, _cacheDependency, countryName );

            if (result == null)
            {
                result = _repository.GetOneByName( countryName );
                _dataCache.SetValue( GetOneByNameSignature, result, countryName );
            }

            return result;
        }

        public void Delete( string countryCode )
        {
            _repository.Delete( countryCode );
            _dataCache.ClearRepositoryCache();
        }

        public IList<Country> GetAll( BoolFilter enabled, string sortBy )
        {
            IList<Country> list = (IList<Country>) _dataCache.GetValue(
                    GetAllSignature,
                    _cacheDependency,
                    enabled.ToString(),
                    sortBy
                    );

            if (list == null)
            {
                list = _repository.GetAll( enabled, sortBy );
                _dataCache.SetValue(
                    GetAllSignature,
                    list,
                    enabled.ToString(),
                    sortBy
                    );
            }

            return list;
        }

        public void DeleteAll()
        {
            _repository.DeleteAll();
            _dataCache.ClearRepositoryCache();
        }

        #endregion
    }
}
