using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Payments;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class PaymentOptionRepository : IPaymentOptionRepository
    {
        #region Private

        private const string RepositoryName = "PatymentMethodRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetLocalesSignature = "GetLocales";
        private const string GetPaymentOptionListSignature = "GetPaymentOptionList";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency =
            new CacheDependencyUtility( new string[] { "PaymentOptionLocale", "PaymentOption" } );

        private PaymentOptionRepositoryNoCache _repository = new PaymentOptionRepositoryNoCache();

        #endregion

        #region Constructors

        public PaymentOptionRepository()
        {
        }

        public PaymentOptionRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Method

        public PaymentOption Update( PaymentOption paymentOption )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            PaymentOption result = _repository.Update( paymentOption );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public PaymentOption GetOne( Culture culture, string name )
        {
            PaymentOption result = (PaymentOption) _dataCache.GetValue(
                GetOneSignature,
                _cacheDependency,
                culture.CultureID,
                name );

            if (result == null)
            {
                result = _repository.GetOne( culture, name );
                _dataCache.SetValue( GetOneSignature, result, culture.CultureID, name );
            }

            return result;
        }

        public IList<ILocale> GetLocales( string name )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache.GetValue( GetLocalesSignature, _cacheDependency, name );

            if (list == null)
            {
                list = _repository.GetLocales( name );
                _dataCache.SetValue( GetLocalesSignature, list, name );
            }

            return list;
        }

        public IList<PaymentOption> GetAll( Culture culture, BoolFilter isEnabled )
        {
            IList<PaymentOption> list = (IList<PaymentOption>) _dataCache.GetValue(
                GetPaymentOptionListSignature, _cacheDependency,
                culture.CultureID,
                isEnabled.ToString(),
                BoolFilter.ShowAll.ToString(),
                BoolFilter.ShowAll.ToString() );

            if (list == null)
            {
                list = _repository.GetAll( culture, isEnabled );
                _dataCache.SetValue(
                    GetPaymentOptionListSignature,
                    list,
                    culture.CultureID,
                    isEnabled.ToString(),
                    BoolFilter.ShowAll.ToString(),
                    BoolFilter.ShowAll.ToString() );
            }

            return list;
        }

        public IList<PaymentOption> GetShownPaymentList( Culture culture, BoolFilter isEnabled )
        {
            IList<PaymentOption> list = (IList<PaymentOption>) _dataCache.GetValue(
                GetPaymentOptionListSignature, _cacheDependency,
                culture.CultureID,
                isEnabled.ToString(),
                BoolFilter.ShowTrue.ToString(),
                BoolFilter.ShowAll.ToString() );

            if (list == null)
            {
                list = _repository.GetShownPaymentList( culture, isEnabled );
                _dataCache.SetValue(
                    GetPaymentOptionListSignature,
                    list,
                    culture.CultureID,
                    isEnabled.ToString(),
                    BoolFilter.ShowTrue.ToString(),
                    BoolFilter.ShowAll.ToString() );
            }

            return list;
        }

        public IList<PaymentOption> GetButtonList( Culture culture, BoolFilter isEnabled )
        {
            IList<PaymentOption> list = (IList<PaymentOption>) _dataCache.GetValue(
                GetPaymentOptionListSignature, _cacheDependency,
                culture.CultureID,
                isEnabled.ToString(),
                BoolFilter.ShowAll.ToString(),
                BoolFilter.ShowTrue.ToString() );

            if (list == null)
            {
                list = _repository.GetButtonList( culture, isEnabled );
                _dataCache.SetValue(
                    GetPaymentOptionListSignature,
                    list,
                    culture.CultureID,
                    isEnabled.ToString(),
                    BoolFilter.ShowAll.ToString(),
                    BoolFilter.ShowTrue.ToString() );
            }

            return list;
        }

        #endregion
    }
}
