using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

using Vevo.Domain.Orders;

namespace Vevo.Domain.Payments.SessionState
{
    public class Session : DictionaryBase
    {
        private string _ticket;
        private DateTime _lastAccessTime;        

        public Session()
        {
            _ticket = Guid.NewGuid().ToString().ToLower();
            _lastAccessTime = DateTime.Now;            
        }


        #region Producted
        protected override void OnInsert( object key, object value )
        {
            if (key.GetType() != typeof( System.String ))
                throw new ArgumentException( "key must be of type String.", "key" );
        }

        protected override void OnRemove( object key, object value )
        {
            if (key.GetType() != typeof( System.String ))
                throw new ArgumentException( "key must be of type String.", "key" );
        }

        protected override void OnSet( object key, object oldValue, object newValue )
        {
            if (key.GetType() != typeof( System.String ))
                throw new ArgumentException( "key must be of type String.", "key" );
        }

        protected override void OnValidate( object key, object value )
        {
            if (key.GetType() != typeof( System.String ))
                throw new ArgumentException( "key must be of type String.", "key" );
        }
        #endregion


        #region Public
        public string Ticket
        {
            get { return _ticket; }
            set { _ticket = value; }
        }

        public object this[string key]
        {
            get
            {
                return Dictionary[key];
            }
            set
            {
                Dictionary[key] = value;
            }
        }

        public ICollection Keys
        {
            get
            {
                return (Dictionary.Keys);
            }
        }

        public ICollection Values
        {
            get
            {
                return (Dictionary.Values);
            }
        }

        public void Add( string key, object value )
        {
            Dictionary.Add( key, value );
        }

        public bool Contains( string key )
        {
            return (Dictionary.Contains( key ));
        }

        public void Remove( string key )
        {
            Dictionary.Remove( key );
        }

        public bool IsExpired( int idleTimeoutSeconds )
        {
            TimeSpan lastAccess = DateTime.Now.Subtract( _lastAccessTime );
            return (lastAccess.CompareTo( new TimeSpan( 0, 0, idleTimeoutSeconds ) ) >= 0);
        }

        public void RenewTimeout()
        {
            _lastAccessTime = DateTime.Now;
        }
        #endregion

    }
}
