﻿using System.Collections.Generic;
using System.IO;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.ImportExport
{
    public class CategoryExportHelper
    {
        private string _tableName;

        private string GenerateHeader()
        {
            return "\"" + _tableName + "id\"\t\"parentid\"\t\"rootid\"\t\"name\"\t\"description\"\t\"image\"\t\"imagealt\"\t" +
                   "\"imagetitle\"\t\"isenabled\"\t\"metakeyword\"\t\"metadescription\"\t\"other1\"\t\"other2\"\t\"other3\"\t\"other4\"\t\"other5\"\t" +
                   "\"isanchor\"\t\"isshownewarrival\"\t\"newarrivalamount\"";
        }

        private string GenerateCategoryDetail( Category category )
        {
            string categoryID = category.CategoryID;
            string description = category.Description;
            string imageAlt = category.ImageAlt;
            string imageFile = category.ImageFile;
            string imageTitle = category.ImageTitle;
            string isAnchor = ConvertUtilities.ToString( category.IsAnchor );
            string isEnabled = ConvertUtilities.ToString( category.IsEnabled );
            string isShowNewArrival = ConvertUtilities.ToString( category.IsShowNewArrival );
            string metaDescription = category.MetaDescription;
            string metaKeyword = category.MetaKeyword;
            string name = category.Name;
            string newArrivalAmount = category.NewArrivalAmount.ToString();
            string other1 = category.Other1;
            string other2 = category.Other2;
            string other3 = category.Other3;
            string other4 = category.Other4;
            string other5 = category.Other5;
            string parentCategoryID = category.ParentCategoryID;
            string rootID = category.RootID;

            return categoryID + "\t" + parentCategoryID + "\t" + rootID + "\t" + name + "\t" + description + "\t" + imageFile + "\t" + imageAlt + "\t" +
                imageTitle + "\t" + isEnabled + "\t" + metaKeyword + "\t" + metaDescription + "\t" + other1 + "\t" + other2 + "\t" + other3 + "\t" + other4 + "\t" + other5 + "\t" +
                isAnchor + "\t" + isShowNewArrival + "\t" + newArrivalAmount;

        }

        private string GenerateDepartmentDetail( Department department )
        {
            string departmentID = department.DepartmentID;
            string description = department.Description;
            string imageAlt = department.ImageAlt;
            string imageFile = department.ImageFile;
            string imageTitle = department.ImageTitle;
            string isAnchor = ConvertUtilities.ToString( department.IsAnchor );
            string isEnabled = ConvertUtilities.ToString( department.IsEnabled );
            string isShowNewArrival = ConvertUtilities.ToString( department.IsShowNewArrival );
            string metaDescription = department.MetaDescription;
            string metaKeyword = department.MetaKeyword;
            string name = department.Name;
            string newArrivalAmount = department.NewArrivalAmount.ToString();
            string other1 = department.Other1;
            string other2 = department.Other2;
            string other3 = department.Other3;
            string other4 = department.Other4;
            string other5 = department.Other5;
            string parentDepartmentID = department.ParentDepartmentID;
            string rootID = department.RootID;

            return departmentID + "\t" + parentDepartmentID + "\t" + rootID + "\t" + name + "\t" + description + "\t" + imageFile + "\t" + imageAlt + "\t" +
                imageTitle + "\t" + isEnabled + "\t" + metaKeyword + "\t" + metaDescription + "\t" + other1 + "\t" + other2 + "\t" + other3 + "\t" + other4 + "\t" + other5 + "\t" +
                isAnchor + "\t" + isShowNewArrival + "\t" + newArrivalAmount;

        }

        public CategoryExportHelper( string tableName )
        {
            _tableName = tableName;
        }

        public MemoryStream ExportToStream( IList<Category> categoryList )
        {
            //Create the return memorystream object that will hold the buffered data.
            MemoryStream returnStream = new MemoryStream();

            //Create a streamwriter to write to the memory stream
            StreamWriter sw = new StreamWriter( returnStream, System.Text.Encoding.Unicode );

            if (categoryList.Count > 0)
            {
                sw.WriteLine( GenerateHeader() );
                string details;
                foreach (Category category in categoryList)
                {
                    details = GenerateCategoryDetail( category );
                    sw.WriteLine( details );
                }
            }

            //Clean up the stream writer
            sw.Flush();
            //sw.Close();

            return returnStream;
        }

        public MemoryStream ExportToStream( IList<Department> departmentList )
        {
            //Create the return memorystream object that will hold the buffered data.
            MemoryStream returnStream = new MemoryStream();

            //Create a streamwriter to write to the memory stream
            StreamWriter sw = new StreamWriter( returnStream, System.Text.Encoding.Unicode );

            if (departmentList.Count > 0)
            {
                sw.WriteLine( GenerateHeader() );
                string details;
                foreach (Department department in departmentList)
                {
                    details = GenerateDepartmentDetail( department );
                    sw.WriteLine( details );
                }
            }

            //Clean up the stream writer
            sw.Flush();
            //sw.Close();

            return returnStream;
        }
    }
}
