﻿using System;
using System.Collections;
using System.Text;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.ImportExport
{
    public class CategoryImportHelperStatus
    {
        public enum ErrorStatus { Success, DoneWithErrors, Error, Fatal }
        private StringBuilder _errorBuilder = new StringBuilder();
        private ErrorStatus _status = ErrorStatus.Success;

        private string _tableName;

        public ErrorStatus Status
        {
            get
            {
                return _status;
            }
        }

        public int ImportedCount;
        public int UpdatedCount;
        public int ErrorCount;
        public int SkippedCount;

        public ArrayList OtherErrorList;
        public ArrayList OtherErrorExceptionList;
        public string OtherError;
        public string FieldError;
        public ArrayList FieldErrorList;
        public string RootError;
        public ArrayList RootErrorList;
        public string ParentContainProductError;
        public ArrayList ParentContainProductErrorList;
        public string RootChangedError;
        public ArrayList RootChangedErrorList;

        public string ErrorMessage
        {
            get
            {
                if (ErrorCount > 0 || _status == ErrorStatus.Fatal)
                    return _errorBuilder.ToString();
                else
                    return String.Empty;
            }
        }

        public CategoryImportHelperStatus( string tableName )
        {
            _tableName = tableName;
            ImportedCount = 0;
            UpdatedCount = 0;
            ErrorCount = 0;
            SkippedCount = 0;
            OtherErrorList = new ArrayList();
            OtherErrorExceptionList = new ArrayList();
            OtherError = String.Empty;
            FieldError = String.Empty;
            FieldErrorList = new ArrayList();
            RootError = String.Empty;
            RootErrorList = new ArrayList();
            ParentContainProductError = String.Empty;
            ParentContainProductErrorList = new ArrayList();
            RootChangedError = String.Empty;
            RootChangedErrorList = new ArrayList();
        }

        private string CreateOtherErrorDetails()
        {
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < 5 && i < OtherErrorExceptionList.Count; i++)
            {
                if (i > 0)
                    sb.Append( "<br/>----------<br/>" );

                Exception ex = (Exception) OtherErrorExceptionList[i];

                sb.Append( "<br/>Message: " );
                sb.Append( ex.Message );
                sb.Append( "<br/>Exception Type: " );
                sb.Append( ex.GetType().ToString() );
                sb.Append( "<br/>Stack Trace:<br/>" );
                sb.Append( HtmlFormatter.ConvertNewLines( ex.StackTrace ) );
            }

            return sb.ToString();
        }

        public void AddSuccess()
        {
            ImportedCount++;
        }

        public void UpdateSuccess()
        {
            UpdatedCount++;
        }

        public void Skip()
        {
            SkippedCount++;
        }

        public void AddError()
        {
            ErrorCount++;
        }

        public void AddOtherError( string lineNumber, Exception ex )
        {
            OtherErrorList.Add( lineNumber );
            OtherErrorExceptionList.Add( ex );

            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddFieldError( string lineNumber )
        {
            FieldErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;
            ErrorCount = 1;
        }

        public void SetFatalError( string message )
        {
            _errorBuilder = new StringBuilder( message );
            _status = ErrorStatus.Fatal;
        }

        public void AddRootError( string lineNumber )
        {
            RootErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;
            AddError();
        }

        public void AddParentContainProductError( string lineNumber )
        {
            ParentContainProductErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;
            AddError();
        }

        public void AddRootChangedError( string lineNumber )
        {
            RootChangedErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;
            AddError();
        }

        public void BuildOtherError()
        {
            if (OtherErrorList.Count != 0)
            {
                string[] result = new string[OtherErrorList.Count];
                OtherErrorList.CopyTo( result );
                OtherError = String.Format(
                    "Other error(s) in the following line number(s) in your csv file:<br />{0}<br/>" +
                    "Details of the first few error(s):<br/>{1}",
                    String.Join( ", ", result ),
                    CreateOtherErrorDetails() );
            }
        }

        public void BuildFieldError()
        {
            if (FieldErrorList.Count != 0)
            {
                string[] result = new string[FieldErrorList.Count];
                FieldErrorList.CopyTo( result );
                FieldError = String.Format(
                    "Some " + _tableName + " fields are invalid.<br/>" +
                    "Please verify " + _tableName + " of the following line number(s) in your csv file:<br />{0}<br/>",
                    String.Join( ", ", result ) );
            }
        }

        public void BuildRootError()
        {
            if (RootErrorList.Count != 0)
            {
                string[] result = new string[RootErrorList.Count];
                RootErrorList.CopyTo( result );
                RootError = String.Format(
                    "Some RootID are invalid.<br/>" +
                    "Please verify Root " + _tableName + " IDs of the following line number(s) in your csv file:<br />{0}<br/>",
                    String.Join( ", ", result ) );
            }
        }

        public void BuildParentContainProductError()
        {
            if (ParentContainProductErrorList.Count != 0)
            {
                string[] result = new string[ParentContainProductErrorList.Count];
                ParentContainProductErrorList.CopyTo( result );
                ParentContainProductError = String.Format(
                    "Some Parent " + _tableName + " are contain Product.<br/>" +
                    "Please verify Parent " + _tableName + " of the following line number(s) in your csv file:<br />{0}<br/>",
                    String.Join( ", ", result ) );
            }
        }

        public void BuildRootChangedError()
        {
            if (RootChangedErrorList.Count != 0)
            {
                string[] result = new string[RootChangedErrorList.Count];
                RootChangedErrorList.CopyTo( result );
                RootChangedError = String.Format(
                    "Cannot change Root " + _tableName + " of the existing Category.<br/>" +
                    "Please verify Root " + _tableName + " IDs of the following line number(s) in your csv file:<br />{0}<br/>",
                    String.Join( ", ", result ) );
            }
        }
    }
}
