﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Text;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.ImportExport
{
    public class ProductExporter
    {
        private const int column_categoryIDIndex = 0;
        private const int column_departmentIDIndex = 1;
        private const int column_nameIndex = 2;
        private const int column_shortDescriptionIndex = 3;
        private const int column_longDescriptionIndex = 4;
        private const int column_skuIndex = 5;
        private const int column_manufacturerIndex = 6;
        private const int column_modelIndex = 7;
        private const int column_manufacturerPartNumber = 8;
        private const int column_upc = 9;
        private const int column_useInventory = 10;
        private const int column_stockIndex = 11;
        private const int column_weightIndex = 12;
        private const int column_priceIndex = 13;
        private const int column_retailPriceIndex = 14;
        private const int column_wholesalePrice1Index = 15;
        private const int column_wholesalePrice2Index = 16;
        private const int column_wholesalePrice3Index = 17;
        private const int column_keywordsIndex = 18;
        private const int column_other1Index = 19;
        private const int column_other2Index = 20;
        private const int column_other3Index = 21;
        private const int column_other4Index = 22;
        private const int column_other5Index = 23;
        private const int column_isEnabledIndex = 24;
        private const int column_taxClassID = 25;

        private const int column_isTodaySpecialIndex = 26;
        private const int column_isDownloadableIndex = 27;
        private const int column_downloadPathIndex = 28;
        private const int column_relatedProducts = 29;

        private const int column_largeImage1 = 30;
        private const int column_isZoom1 = 31;
        private const int column_isEnlarge1 = 32;
        private const int column_largeImage2 = 33;
        private const int column_isZoom2 = 34;
        private const int column_isEnlarge2 = 35;
        private const int column_largeImage3 = 36;
        private const int column_isZoom3 = 37;
        private const int column_isEnlarge3 = 38;
        private const int column_largeImage4 = 39;
        private const int column_isZoom4 = 40;
        private const int column_isEnlarge4 = 41;
        private const int column_largeImage5 = 42;
        private const int column_isZoom5 = 43;
        private const int column_isEnlarge5 = 44;
        private const int column_largeImage6 = 45;
        private const int column_isZoom6 = 46;
        private const int column_isEnlarge6 = 47;
        private const int column_largeImage7 = 48;
        private const int column_isZoom7 = 49;
        private const int column_isEnlarge7 = 50;
        private const int column_largeImage8 = 51;
        private const int column_isZoom8 = 52;
        private const int column_isEnlarge8 = 53;
        private const int column_largeImage9 = 54;
        private const int column_isZoom9 = 55;
        private const int column_isEnlarge9 = 56;
        private const int column_largeImage10 = 57;
        private const int column_isZoom10 = 58;
        private const int column_isEnlarge10 = 59;
        private const int column_isGiftCertificate = 60;
        private const int column_isFreeShippingCost = 61;
        private const int column_isFixedPrice = 62;

        private const int column_giftAmount = 63;
        private const int column_expiredType = 64;
        private const int column_fixExpiredDate = 65;
        private const int column_numberOfDay = 66;
        private const int column_isElectronic = 67;

        private const int column_useOptionIndex = 68;

        private const int column_optionGroup1IDIndex = 69;
        private const int column_useStock1Index = 70;
        private const int column_optionGroup2IDIndex = 71;
        private const int column_useStock2Index = 72;
        private const int column_optionGroup3IDIndex = 73;
        private const int column_useStock3Index = 74;
        private const int column_optionGroup4IDIndex = 75;
        private const int column_useStock4Index = 76;
        private const int column_optionGroup5IDIndex = 77;
        private const int column_useStock5Index = 78;

        private const int column_optionItem1IDIndex = 79;
        private const int column_optionItem2IDIndex = 80;
        private const int column_optionItem3IDIndex = 81;
        private const int column_optionItem4IDIndex = 82;
        private const int column_optionItem5IDIndex = 83;

        private const int column_isRecurring = 84;
        private const int column_recurringInterval = 85;
        private const int column_recurringIntervalUnit = 86;
        private const int column_recurringNumberOfCycles = 87;
        private const int column_recurringNumberOfTrialCycles = 88;
        private const int column_recurringTrialAmount = 89;

        private const int column_metaKeyword = 90;
        private const int column_metaDescription = 91;

        private const int column_isAffiliate = 92;

        private const int column_minQuantity = 93;
        private const int column_maxQuantity = 94;

        // Product custom price 
        private const int column_isCustomPrice = 95;
        private const int column_defaultPrice = 96;
        private const int column_minimumPrice = 97;

        private const int column_isCallForPrice = 98;
        private const int column_brand = 99;
        private const int column_returnTime = 100;
        private const int column_isProductKit = 101;
        private const int column_isDynamicProductKitPrice = 102;
        private const int column_isDynamicProductKitWeight = 103;
        private const int column_productKitGroupID = 104;
        private const int column_createDate = 105;
        private const int column_viewCount = 106;

        private const int max_columns = 107;

        private string GenerateHeader()
        {
            return "\"CategoryID\",\"DepartmentID\",\"Name\",\"ShortDescription\",\"LongDescription\",\"Sku\",\"Manufacturer\"," +
                "\"Model\",\"ManufacturerPartNumber\",\"Upc\",\"UseInventory\",\"Stock\",\"Weight\",\"Price\",\"RetailPrice\"," +
                "\"WholesalePrice\",\"WholesalePrice2\",\"WholesalePrice3\",\"Keywords\",\"Other1\",\"Other2\",\"Other3\",\"Other4\",\"Other5\"," +
                "\"IsEnabled\",\"IsTaxable\",\"IsTodaySpecial\",\"IsDownloadable\",\"DownloadPath\",\"RelatedProducts\"," +
                "\"LargeImage1\",\"IsZoom1\",\"IsEnlarge1\",\"LargeImage2\",\"IsZoom2\",\"IsEnlarge2\",\"LargeImage3\",\"IsZoom3\",\"IsEnlarge3\"," +
                "\"LargeImage4\",\"IsZoom4\",\"IsEnlarge4\",\"LargeImage5\",\"IsZoom5\",\"IsEnlarge5\",\"LargeImage6\",\"IsZoom6\",\"IsEnlarge6\"," +
                "\"LargeImage7\",\"IsZoom7\",\"IsEnlarge7\",\"LargeImage8\",\"IsZoom8\",\"IsEnlarge8\",\"LargeImage9\",\"IsZoom9\",\"IsEnlarge9\",\"LargeImage10\",\"IsZoom10\",\"IsEnlarge10\"," +
                "\"IsGiftCertificate\",\"IsFreeShippingCost\",\"IsFixedPrice\",\"GiftAmount\",\"ExpireType\",\"FixedExpireDate\"," +
                "\"NumberOfDays\",\"IsElectronic\",\"UseOption\",\"OptionGroupID1\",\"UseStockOption1\",\"OptionGroupID2\",\"UseStockOption2\",\"OptionGroupID3\"," +
                "\"UseStockOption3\",\"OptionGroupID4\",\"UseStockOption4\",\"OptionGroupID5\",\"UseStockOption5\"," +
                "\"OptionItemID1\",\"OptionItemID2\",\"OptionItemID3\",\"OptionItemID4\",\"OptionItemID5\"," +
                "\"IsRecurring\",\"RecurringInterval\",\"RecurringIntervalUnit\",\"RecurringNumberOfCycles\",\"RecurringNumberOfTrialCycles\"," +
                "\"RecurringTrialAmount\",\"MetaKeyword\",\"MetaDescription\",\"IsAffiliate\",\"MinQuantity\",\"MaxQuantity\",\"IsCustomPrice\"," +
                "\"DefaultPrice\",\"MinimumPrice\",\"IsCallForPrice\",\"Brand\",\"ReturnTime\",\"IsProductKit\",\"IsDynamicProductKitPrice\",\"IsDynamicProductKitWeight\",\"ProductKitGroupID\"," +
                "\"CreateDate\",\"ViewCount\"";
        }

        private string convertBoolToCSV( bool value )
        {
            if (value)
                return "Yes";
            else
                return "No";
        }
        
        private string convertRelatedToSKU( string productIDList, Culture culture, string storeID )
        {
            string[] list = productIDList.Split( ",".ToCharArray() );
            string[] skuList = new string[list.Length];
            for (int i = 0; i < list.Length; i++)
            {
                Product product = DataAccessContext.ProductRepository.GetOne( culture, list[i].Trim(), storeID );
                skuList[i] = product.Sku;
            }
            return String.Join( "|", skuList );
        }

        private string[] formatCSVLine( string[] csvLines )
        {
            string[] outArray = new string[max_columns];
            for (int i = 0; i < outArray.Length; i++)
            {
                if (csvLines[i] != null)
                {
                    string ex = csvLines[i];
                    ex = ex.Replace( "\"", "\"\"" );
                    outArray[i] = "\"" + ex + "\"";
                }
            }
            return outArray;
        }

        private string GetPathWithTilde( string relativePath )
        {
            if (relativePath.Trim().StartsWith( "~/" ))
                return relativePath.Trim();
            else
                return "~/" + relativePath.Trim();
        }


        private static bool CopyFiles( string sSource, string sDestn )
        {
            try
            {
                if (File.Exists( sSource ) == true)
                {
                    File.Copy( sSource, sDestn, true );
                    return true;
                }
                else
                {
                    Console.WriteLine( "Specifed file does not exist" );
                    return false;
                }
            }
            catch (FileNotFoundException exFile)
            {
                Console.WriteLine( "File Not Found " + exFile.Message );
                return false;
            }
            catch (DirectoryNotFoundException exDir)
            {
                Console.WriteLine( "Directory Not Found " + exDir.Message );
                return false;
            }
            catch (Exception ex)
            {
                Console.WriteLine( ex.Message );
                return false;
            }
        }

        private string GetFolderImageToExport( string filename )
        {
            string _filename = Path.GetFileName( filename );
            return "Export/Images/" + _filename;
        }

        private string GenerateDetail( Product product, Culture culture, string storeID )
        {
            ArrayList groupIDList = new ArrayList();
            ArrayList optionItemIDList = new ArrayList();
            if (product.ProductOptionGroups.Count > 0)
            {
                foreach (ProductOptionGroup group in product.ProductOptionGroups)
                {
                    if (group.UseStock)
                    {
                        groupIDList.Add( group.OptionGroupID );
                    }
                }
                DataTable dt = DataAccessContext.ProductRepository.GetStockOptionLine( culture, (string[]) groupIDList.ToArray( typeof( string ) ) );

                if (dt != null)
                {
                    foreach (DataRow row in dt.Rows)
                    {
                        ArrayList optionItemIDs = new ArrayList();
                        for (int jm = 0; jm < dt.Columns.Count - 1; jm += 2)
                        {
                            optionItemIDs.Add( row[jm].ToString() );
                        }
                        optionItemIDList.Add( optionItemIDs );
                    }
                }
            }

            string exString = String.Empty;

            string[] columns = new string[max_columns];
            string[] UseStockOptionx = new string[5];
            int temp = 0;
            if (product.ProductOptionGroups.Count > 0)
                columns[column_useOptionIndex] = convertBoolToCSV( true );
            else
                columns[column_useOptionIndex] = convertBoolToCSV( false );
            int startOptionGriupColumn = column_optionGroup1IDIndex;
            foreach (ProductOptionGroup group in product.ProductOptionGroups)
            {
                if (startOptionGriupColumn > column_optionGroup5IDIndex)
                    break;
                columns[startOptionGriupColumn] = group.OptionGroupID;
                columns[startOptionGriupColumn + 1] = convertBoolToCSV( group.UseStock );
                UseStockOptionx[temp] = convertBoolToCSV( group.UseStock );
                startOptionGriupColumn += 2;
                temp++;
            }

            string[] categoryList = new string[product.CategoryIDs.Count];
            product.CategoryIDs.CopyTo( categoryList, 0 );
            columns[column_categoryIDIndex] = String.Join( "|", categoryList );

            if (product.DepartmentIDs.Count > 0)
            {
                string[] departmentList = new string[product.DepartmentIDs.Count];
                product.DepartmentIDs.CopyTo( departmentList, 0 );
                columns[column_departmentIDIndex] = String.Join( "|", departmentList );
            }

            columns[column_nameIndex] = product.Name;
            columns[column_shortDescriptionIndex] = product.ShortDescription;
            columns[column_longDescriptionIndex] = product.LongDescription.Replace( "\n", "<BR>" );
            columns[column_skuIndex] = product.Sku;
            columns[column_manufacturerIndex] = product.Manufacturer;
            columns[column_modelIndex] = product.Model;
            columns[column_manufacturerPartNumber] = product.ManufacturerPartNumber;
            columns[column_upc] = product.Upc;
            columns[column_useInventory] = convertBoolToCSV( product.UseInventory );

            columns[column_weightIndex] = product.Weight.ToString();

            ProductPrice csvPrice = product.GetProductPrice( storeID );
            columns[column_priceIndex] = csvPrice.Price.ToString();
            columns[column_retailPriceIndex] = csvPrice.RetailPrice.ToString();
            columns[column_wholesalePrice1Index] = csvPrice.WholesalePrice.ToString();
            columns[column_wholesalePrice2Index] = csvPrice.WholesalePrice2.ToString();
            columns[column_wholesalePrice3Index] = csvPrice.WholesalePrice3.ToString();

            columns[column_keywordsIndex] = product.Keywords;
            columns[column_other1Index] = product.Other1;
            columns[column_other2Index] = product.Other2;
            columns[column_other3Index] = product.Other3;
            columns[column_other4Index] = product.Other4;
            columns[column_other5Index] = product.Other5;
            columns[column_isEnabledIndex] = convertBoolToCSV( product.IsEnabled );
            columns[column_taxClassID] = product.TaxClassID;
            columns[column_isTodaySpecialIndex] = convertBoolToCSV( product.IsTodaySpecial );
            columns[column_isDownloadableIndex] = convertBoolToCSV( product.IsDownloadable );
            columns[column_downloadPathIndex] = product.DownloadPath;
            columns[column_relatedProducts] = convertRelatedToSKU( product.RelatedProducts, culture, storeID );

            int startImageColumn = column_largeImage1;
            foreach (ProductImage image in product.ProductImages)
            {
                if (startImageColumn > column_largeImage10)
                    break;
                columns[startImageColumn] = image.LargeImage;
                columns[startImageColumn + 1] = convertBoolToCSV( image.IsZoom );
                columns[startImageColumn + 2] = convertBoolToCSV( image.IsEnlarge );
                startImageColumn += 3;
            }

            columns[column_isGiftCertificate] = convertBoolToCSV( product.IsGiftCertificate );
            columns[column_isFreeShippingCost] = convertBoolToCSV( product.FreeShippingCost );
            columns[column_isFixedPrice] = convertBoolToCSV( product.IsFixedPrice );

            columns[column_isRecurring] = convertBoolToCSV( product.IsRecurring );
            if (product.ProductRecurring != null)
            {
                columns[column_recurringInterval] = product.ProductRecurring.RecurringInterval.ToString();
                columns[column_recurringIntervalUnit] = product.ProductRecurring.RecurringIntervalUnit.ToString();
                columns[column_recurringNumberOfCycles] = product.ProductRecurring.RecurringNumberOfCycles.ToString();
                columns[column_recurringNumberOfTrialCycles] = product.ProductRecurring.RecurringNumberOfTrialCycles.ToString();
                columns[column_recurringTrialAmount] = product.ProductRecurring.RecurringTrialAmount.ToString();
            }
            if (product.GetMetaInformation( culture, storeID ) != null)
            {
                columns[column_metaKeyword] = product.GetMetaInformation( culture, storeID ).MetaKeyword;
                columns[column_metaDescription] = product.GetMetaInformation( culture, storeID ).MetaDescription;
            }

            columns[column_isAffiliate] = convertBoolToCSV( product.IsAffiliate );

            columns[column_minQuantity] = product.MinQuantity.ToString();
            columns[column_maxQuantity] = product.MaxQuantity.ToString();

            columns[column_isCustomPrice] = convertBoolToCSV( product.IsCustomPrice );
            if (product.IsCustomPrice)
            {
                columns[column_defaultPrice] = product.ProductCustomPrice.DefaultPrice.ToString();
                columns[column_minimumPrice] = product.ProductCustomPrice.MinimumPrice.ToString();
            }

            columns[column_isCallForPrice] = convertBoolToCSV( product.IsCallForPrice );

            columns[column_brand] = product.Brand;
            columns[column_returnTime] = product.ReturnTime.ToString();
            columns[column_isProductKit] = convertBoolToCSV( product.IsProductKit );
            columns[column_isDynamicProductKitPrice] = convertBoolToCSV( product.IsDynamicProductKitPrice );
            columns[column_isDynamicProductKitWeight] = convertBoolToCSV( product.IsDynamicProductKitWeight );
            columns[column_createDate] = ConvertUtilities.ToString( product.CreateDate );
            columns[column_viewCount] = product.ViewCount.ToString();

            if (product.ProductKits.Count > 0)
            {
                string[] kitGroupIDs = new string[product.ProductKits.Count];
                for (int i = 0; i < product.ProductKits.Count; i++)
                {
                    if (i == 5)
                        break;
                    kitGroupIDs[i] = product.ProductKits[i].ProductKitGroupID;
                }
                columns[column_productKitGroupID] = String.Join( "|", kitGroupIDs );
            }

            if (optionItemIDList.Count > 0)
            {
                foreach (ArrayList optionItemIDs in optionItemIDList)
                {
                    int startOptionItemIDColumn = column_optionItem1IDIndex;
                    int x = 0;
                    for (int i = 0; i < 5; i++)
                    {
                        if (UseStockOptionx[i] == "Yes")
                        {
                            columns[startOptionItemIDColumn + i] = optionItemIDs[x].ToString();
                            x++;
                        }
                    }
                    columns[column_stockIndex] = product.GetStock( (string[]) optionItemIDs.ToArray( typeof( string ) ) ).ToString();


                    exString += String.Join( ",", formatCSVLine( columns ) ) + Environment.NewLine;
                }
                exString = exString.TrimEnd( Environment.NewLine.ToCharArray() );
            }
            else
            {
                columns[column_stockIndex] = product.SumStock.ToString();
                exString += String.Join( ",", formatCSVLine( columns ) );
            }

            return exString;
        }

        public MemoryStream ExportToStream( IList<Product> productsList, Culture culture, string storeID )
        {
            //Create the return memorystream object that will hold the buffered data.
            MemoryStream returnStream = new MemoryStream();

            //Create a streamwriter to write to the memory stream
            StreamWriter sw = new StreamWriter( returnStream, Encoding.Default );

            if (productsList.Count > 0)
            {
                sw.WriteLine( GenerateHeader() );
                string details;
                foreach (Product product in productsList)
                {
                    details = GenerateDetail( product, culture, storeID );
                    sw.WriteLine( details );
                }
            }

            //Clean up the stream writer
            sw.Flush();
            //sw.Close();

            return returnStream;
        }
    }
}
