using System;
using System.Data;
using System.Configuration;
using Vevo.Shared.DataAccess;

namespace Vevo.Domain
{
    /// <summary>
    /// Summary description for SystemConst
    /// </summary>
    ///     
    public static class SystemConst
    {
        public static string CurrentVersion()
        {
            if (KeyUtilities.IsTrialLicense())
            {
                return "Trial";
            }
            else if (KeyUtilities.IsServerLicense())
            {
                return "Server";
            }
            else if (KeyUtilities.IsMultistoreLicense())
            {
                return "Multistore";
            }
            else if (KeyUtilities.IsDeluxeLicense( DataAccessHelper.DomainRegistrationkey, DataAccessHelper.DomainName ))
            {
                return "Deluxe";
            }
            else
            {

                return "Community";
            }
        }

        public static string CurrentVevoCartVersionNumber()
        {
            return "VevoCart " + CurrentVersion() + " Ver. " + CurrentVersionNumber;
        }

        public const string CurrentVersionNumber = "6.0.1";
        public const string CurrentMajorVersionNumber = "6";
        
        public const bool IsReleasedVersion = true;

        //-------------------------------------------------------------------
        // Admin
        //-------------------------------------------------------------------
        public const string AdminDefaultCultureName = "en-US";

        //-------------------------------------------------------------------
        // File locations
        //-------------------------------------------------------------------
        public const string LicenseFilePath = "~/LicenseKey.xml";
        public const string FreeTextBoxDirectory = "~/ClientScripts/FreeTextBox/";

        //-------------------------------------------------------------------
        // Sidebars
        //-------------------------------------------------------------------
        public const string MaximumWidthLeftSidebarImage = "218px";
        public const string MaximumWidthRightSidebarImage = "218px";

        //-------------------------------------------------------------------
        // Product Display
        //-------------------------------------------------------------------
        public const int ProductMagnifierSize = 200;
        public const string FormarDateTime = "MMM dd, yyyy";

        //-------------------------------------------------------------------
        // Product Options
        //-------------------------------------------------------------------
        public const string OptionFileUpload = "Images/UploadFile/";
        public const string OptionFileTempPath = "Images/UploadFileTemp/";
        public const string OptionInputList = "Input List";

        //-------------------------------------------------------------------
        // Page Layout
        //-------------------------------------------------------------------
        public const string LayoutCategoryListPath = "~/Layouts/CategoryLists/";
        public const string LayoutDepartmentListPath = "~/Layouts/DepartmentLists/";
        public const string LayoutManufacturerListPath = "~/Layouts/ManufacturerLists/";
        public const string LayoutProductListPath = "~/Layouts/ProductLists/";
        public const string LayoutProductDetailsPath = "~/Layouts/ProductDetails/";
        public const string LayoutBlogListPath = "~/Blog/Layouts/BlogLists/";
        public const string LayoutBlogDetailsPath = "~/Blog/Layouts/BlogDetails/";

        //-------------------------------------------------------------------
        // Install
        //-------------------------------------------------------------------
        public const string ApplicationSettingsFilePath = "~/App_Data/ApplicationSettings.config";

        //-------------------------------------------------------------------
        // Others
        //-------------------------------------------------------------------
        public const string DefaultShippingMethod = "Store Shipping";
        public const string UnknownUser = "Unknown User";
        public const string AnonymousUser = "Anonymous User";
        public const string USCultureName = "en-US";

        public const string DefaultUrlCultureName = "en-US";

        public const int WaitDatabaseConfigUpgrade = 5000;      // Unit in msec

        public const int UnlimitedNumber = 999999;
        public const double UnlimitedNumberDouble = 999999.0;
        public const decimal UnlimitedNumberDecimal = 99999999999.99M;

        public const int MaxConnectionStringLength = 128;
        public const string ConnectionStringAccessFilePath = "App_Data/ConnectionStrings_access.config";
        public const string ConnectionStringSQLFilePath = "App_Data/ConnectionStrings_sql.config";

        public enum DiscountType
        {
            Step,
            Fix
        }

        // RecurringProfile status. PayPal will send IPN to update these status
        //      New            -> Order is just created
        //      CreateVerified -> PayPal sends IPN type "recurring_payment_profile_created"
        //      Ongoing        -> Payment is chared. IPN type "recurring_payment"
        //                        with "RecurringStatus : Active" set in IPN.
        //      Expired        -> The last recurring has been passed. IPN type "recurring_payment" 
        //                        with "RecurringStatus : Expired" set in IPN.
        //      Canceled       -> 
        public enum RecurringStatus
        {            
            Canceled,            
            CreateVerified,
            Expired,
            New,
            Ongoing
        }

        public const int DecimalPointLenght = 2;

        public const string CategoryTreeViewLeftKey = "CategoryTreeViewLeft";
        public const string DepartmentTreeViewLeftKey = "DepartmentTreeViewLeft";

        public const string MobileViewBrowserCookieName = "MobileView";
        public const string IsViewInMobileSessionName = "IsViewMobile";

        public const int EBayUpdateLimitTimeRange = 30; // unit is minutes
        public const int UploadSizeConfigFactor = 1024;
    }

}
