using System;
using Vevo.Domain;
using Vevo.QuickBooks.Qbxml70;

namespace Vevo.QuickBooks.States
{
    /// <summary>
    /// Summary description for ProductAddResponseState
    /// </summary>
    public class ProductAddResponseState : QBWCState
    {
        private void MoveToNextState()
        {
            QBWCSession.CurrentState = new OrderAddRequestState( QBWCSession );
        }

        private void ProcessRecieve( string response )
        {
            QBXMLMsgsRs qbXMLMsgsRs = CreateResponse( response );

            int successCount = 0;
            int failureCount = 0;

            for (int i = 0; i < qbXMLMsgsRs.Items.Length; i++)
            {
                ItemNonInventoryAddRsType result = (ItemNonInventoryAddRsType) qbXMLMsgsRs.Items[i];

                string productID = string.Empty;
                if (result.statusCode.ToString() == "0")
                {
                    SalesOrPurchase sales = (SalesOrPurchase) result.ItemNonInventoryRet.Item;
                    string[] desc = sales.Desc.Split( ' ' );
                    productID = desc[1];
                    DataAccessContext.ProductRepository.UpdateQBData( productID, result.ItemNonInventoryRet.ListID, true );
                    successCount++;
                }
                else
                {
                    CreateQBLogItem( result.statusCode, result.statusSeverity, result.statusMessage, QBExportType.ProductAdd );
                    failureCount++;
                }
            }

            CreateQBLogSummary( QBExportType.ProductAdd, successCount, failureCount );
        }

        public ProductAddResponseState( QBWCSession session )
            : base( session )
        {
        }

        public override int OnReceiveResponseXml( string responseXml )
        {
            ProcessRecieve( responseXml );
            QBWCSession.ClearList();
            MoveToNextState();

            return 64;
        }
    }
}