using System;
using System.Data;
using System.IO;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Vevo.Domain;
using Vevo.Shared.WebUI;
using Vevo.WebAppLib;
using Vevo.WebUI.Themes;
using Vevo.Domain.Stores;

namespace Vevo.WebUI.BaseControls
{
    /// <summary>
    /// Summary description for BasePage
    /// </summary>
    public class BasePage : Page
    {
        private string GetXmlPath()
        {
            if (UrlManager.IsMobile())
            {
                return HttpContext.Current.Server.MapPath(StoreTheme.GetXmlPath(
                    DataAccessContext.Configurations.GetValueNoThrow("MobileTheme")));
            }
            else if (UrlManager.IsFacebook())
            {
                return HttpContext.Current.Server.MapPath(StoreTheme.GetXmlPath("FacebookTheme"));
            }
            else if (UrlManager.IsBlog())
            {
                return HttpContext.Current.Server.MapPath(StoreTheme.GetXmlPath(
                    DataAccessContext.Configurations.GetValueNoThrow("BlogTheme")));
            }
            else
            {
                return HttpContext.Current.Server.MapPath(StoreTheme.GetXmlPath(
                    DataAccessContext.Configurations.GetValueNoThrow("StoreTheme")));
            }
        }

        private StoreTheme GetStoreTheme()
        {
            if (!File.Exists(GetXmlPath()))
                return null;

            StoreThemeFactory storeThemeFactory = new StoreThemeFactory();
            StoreTheme storeTheme = null;
            TextReader reader = null;
            try
            {
                reader = new StreamReader(GetXmlPath());
                storeTheme = storeThemeFactory.CreateThemeFromXml(reader);
            }
            finally
            {
                reader.Close();
            }

            return storeTheme;
        }

        private string CurrentUrl
        {
            get
            {
                return Request.Url.AbsoluteUri;
            }
        }

        private bool IsAdminPath()
        {
            UrlPath path = new UrlPath(CurrentUrl);
            if (CurrentUrl.ToLower().Contains(DataAccessContext.Configurations.GetValue("AdminAdvancedFolder").ToLower()))
            {
                return true;
            }

            return false;
        }

        public void AddMetaTag(string name, string content)
        {
            WebUtilities.PageAddMeta(this, name, content);
        }

        protected void Page_PreInit( object sender, EventArgs e )
        {
            // This is necessary because Safari and Chrome browsers don't display the Menu control correctly.
            // All webpages displaying an ASP.NET menu control must inherit this class.
            if (Request.ServerVariables["http_user_agent"] != null &&
                Request.ServerVariables["http_user_agent"].IndexOf("Safari", StringComparison.CurrentCultureIgnoreCase) != -1)
                Page.ClientTarget = "uplevel";

            StoreRetriever storeRetriever = new StoreRetriever();
            Store store = storeRetriever.GetStore();

            if (!store.IsEnabled && !IsAdminPath())
            {
                Response.Redirect("~/SiteMaintenance.aspx", true);
            }

            StoreTheme storeTheme = GetStoreTheme();
            if (storeTheme != null && Page.Master != null)
            {
                UrlPath urlPath = new UrlPath(CurrentUrl);
                Page.MasterPageFile = storeTheme.GetMasterPagePath(urlPath.ExtractRelativePathFromApplicationFolder());
            }
        }

        public override string StyleSheetTheme
        {
            get
            {

                if (UrlManager.IsMobile())
                {
                    return DataAccessContext.Configurations.GetValueNoThrow("MobileTheme");
                }
                else if (UrlManager.IsFacebook())
                {
                    return "FacebookTheme";
                }
                else if (UrlManager.IsBlog())
                {
                    return DataAccessContext.Configurations.GetValueNoThrow("BlogTheme");
                }
                else
                {
                    return DataAccessContext.Configurations.GetValueNoThrow("StoreTheme");
                }
            }
        }
    }
}
