using System;
using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Web;
using System.Web.Caching;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Vevo.DataAccessLib;
using Vevo.Domain;
using Vevo.WebAppLib;


namespace Vevo.WebUI.DataAccessCache
{
    public interface ILanguageTextAccessCache
    {
        DataTable GetTextByPage( string pageID, string cultureID );
    }


    /// <summary>
    /// Summary description for LanguageTextAccessCache
    /// </summary>
    public static class LanguageTextAccessCache 
    {
        private static CachedObject _cachedObject = new CachedObject();
        private static NonCachedObject _nonCachedObject = new NonCachedObject();


        public static ILanguageTextAccessCache Instance
        {
            get
            {
                if (DataAccessContext.Configurations.GetBoolValueNoThrow( "LanguageCachingEnabled" ))
                    return _cachedObject;
                else
                    return _nonCachedObject;
            }
        }

        public static void ClearCache()
        {
            _cachedObject.ClearCache();
            
        }


        //----------------------------------------------------------------------------------
        // CachedObject
        //----------------------------------------------------------------------------------

        private class CachedObject : BaseDataAccessCache, ILanguageTextAccessCache
        {
            private const string PageTextCacheName = "LanguageTextAccessCache_PageIDToText_";


            public CachedObject()
                : base( "LanguageTextAccessCache" )
            {
            }

            public override void ClearCache()
            {
                base.ClearCache();    
                WebUtilities.ClearCacheWithPrefix( PageTextCacheName );
            }

            public DataTable GetTextByPage( string pageID, string cultureID )
            {
                string key = PageTextCacheName + GenerateKey( pageID, cultureID );

                DataTable table = (DataTable) HttpRuntime.Cache[key];

                if (table == null)
                {
                    table = LanguageTextAccess.GetTextByPage( pageID, cultureID );
                    HttpRuntime.Cache[key] = table;
                }

                return table;
            }
        }

        //----------------------------------------------------------------------------------
        // NonCachedObject
        //----------------------------------------------------------------------------------

        private class NonCachedObject : ILanguageTextAccessCache
        {
            public DataTable GetTextByPage( string pageID, string cultureID )
            {
                return LanguageTextAccess.GetTextByPage( pageID, cultureID );
            }

        }

    }
}
