using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using Vevo.Domain;
using Vevo.Domain.Orders;
using Vevo.Domain.Products;
using Vevo.Domain.Stores;
using Vevo.Shared.Utilities;
using Vevo.Deluxe.Domain.BundlePromotion;
using Vevo.Deluxe.Domain.Products;

namespace Vevo.WebUI.Products
{
    /// <summary>
    /// Summary description for BaseProductUserControl
    /// </summary>
    public partial class BaseProductUserControl : Vevo.WebUI.International.BaseLanguageUserControl
    {
        #region Private

        private Control _optionGroupDetailsParent;
        private Control _addToCartNotificationParent;

        private bool IsOutOfStock( string productID, int purchaseQuantity, out int currentStock )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( StoreContext.Culture, productID, new StoreRetriever().GetCurrentStoreID() );
            currentStock = product.GetPurchasableStock(
                 StoreContext.ShoppingCart.GetNumberOfItems( productID ) + PromotionSelectedItem.FindSubProductAmountInPromotion(StoreContext.ShoppingCart,productID));
            return product.IsOutOfStock(
                purchaseQuantity,
                StoreContext.ShoppingCart.GetNumberOfItems( productID ) + PromotionSelectedItem.FindSubProductAmountInPromotion( StoreContext.ShoppingCart, productID ) );
        }

        private bool IsOutOfStock( string productID, int purchaseQuantity, out int currentStock, out string errorCurrentStock )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( StoreContext.Culture, productID, new StoreRetriever().GetCurrentStoreID() );
            currentStock = product.GetPurchasableStock(
                 StoreContext.ShoppingCart.GetNumberOfItems( productID ) + PromotionSelectedItem.FindSubProductAmountInPromotion( StoreContext.ShoppingCart, productID ) );
            if (product.IsOutOfStock(
                purchaseQuantity,
                StoreContext.ShoppingCart.GetNumberOfItems( productID ) + PromotionSelectedItem.FindSubProductAmountInPromotion( StoreContext.ShoppingCart, productID ) ))
            {
                errorCurrentStock = product.Name;
                return true;
            }
            else
            {
                errorCurrentStock = String.Empty;
                return false;
            }
        }

        private bool VerifyStockOption(
            string productID, string[] optionItemIDs, int quantity, out int currentStock, string storeID )
        {
            return VerifyStockOption( productID, optionItemIDs, quantity, String.Empty, out currentStock, storeID );
        }

        private bool VerifyStockOption(
            string productID, string[] optionItemIDs, int qty, string name, out int currentStock, string storeID )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( StoreContext.Culture, productID, storeID );

            if (optionItemIDs == null || optionItemIDs.Length == 0)
            {
                if (IsOutOfStock( productID, qty, out currentStock ))
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }
            else
            {
                string combinationID = product.FindProductStock( optionItemIDs ).OptionCombinationID;
                currentStock = product.GetPurchasableStock(
                    StoreContext.ShoppingCart.GetNumberOfItems( productID, combinationID ) + PromotionSelectedItem.FindSubProductAmountInPromotion( StoreContext.ShoppingCart, productID, combinationID ),
                    optionItemIDs );
                return !product.IsOutOfStock(
                    qty,
                    StoreContext.ShoppingCart.GetNumberOfItems( productID, combinationID ) + PromotionSelectedItem.FindSubProductAmountInPromotion( StoreContext.ShoppingCart, productID, combinationID ),
                    optionItemIDs
                    );
            }
        }

        private bool VerifyStockOptionWithInputList(
            string productID,
            OptionItemValueCollection inputList,
            OptionItemValueCollection optionsWithoutInputList,
            out string errorCurrentStock,
            out int currentStock
            )
        {
            bool verify = true;
            errorCurrentStock = String.Empty;
            currentStock = 0;
            foreach (OptionItemValue input in inputList)
            {
                OptionItemValueCollection optionsWithStock = optionsWithoutInputList.GetUseStockOptionItems();

                if (input.UseStock)
                {
                    optionsWithStock.Add( input );
                }

                if (optionsWithStock.Count > 0)
                {
                    if (!VerifyStockOption(
                        productID,
                        optionsWithStock.ConvertToIDArray(),
                        ConvertUtilities.ToInt32( input.Details ),
                        input.Name,
                        out currentStock, new StoreRetriever().GetCurrentStoreID() ))
                    {
                        errorCurrentStock = input.Name;
                        verify = false;
                        break;
                    }
                }
            }

            return verify;
        }

        private Product GetProduct( Control bindingContainer )
        {
            return (Product) DataBinder.GetDataItem( bindingContainer );
        }

        #endregion


        #region Protected

        protected Control OptionGroupDetailsParent
        {
            get
            {
                if (_optionGroupDetailsParent == null)
                    _optionGroupDetailsParent = this;

                return _optionGroupDetailsParent;
            }
            set
            {
                _optionGroupDetailsParent = value;
            }
        }

        protected Control AddToCartNotificationParent
        {
            get
            {
                if (_addToCartNotificationParent == null)
                    _addToCartNotificationParent = this;

                return _addToCartNotificationParent;
            }
            set
            {
                _addToCartNotificationParent = value;
            }
        }

        protected IOptionGroupDetailsControl uxOptionGroupDetails
        {
            get
            {
                return (IOptionGroupDetailsControl) OptionGroupDetailsParent.FindControl( "uxOptionGroupDetails" );
            }
        }

        protected IAddToCartNotificationControl uxAddToCartNotification
        {
            get
            {
                IAddToCartNotificationControl control =
                    (IAddToCartNotificationControl) AddToCartNotificationParent.FindControl( "uxAddToCartNotification" );

                if (control != null)
                    return control;
                else
                    return (IAddToCartNotificationControl) AddToCartNotificationParent;
            }
        }

        protected bool AddToCartByAdmin(
            Product product,
            OptionItemValueCollection selectedOptions,
            int quantity,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice,
            decimal enterPrice,
            string storeID,
            out string errorCurrentStock,
            out int currentStock,
            bool isSubscriptionable )
        {
            errorCurrentStock = String.Empty;
            bool result = true;
            if (selectedOptions == null || selectedOptions.Count == 0)
            {
                if (IsOutOfStock( product.ProductID, quantity, out currentStock ))
                {
                    result = false;
                }
                else
                {
                    if (product.IsGiftCertificate)
                    {
                        //StoreContext.ShoppingCart.AddItem(
                        //    product, quantity, giftDetails );

                        StoreContext.ShoppingCart.AddItemByAdmin( product, quantity, OptionItemValueCollection.Null, giftDetails, 0, 0, isSubscriptionable );
                    }
                    else if (product.IsCustomPrice)
                    {
                        //StoreContext.ShoppingCart.AddItem( product, quantity, productCustomPrice );
                        StoreContext.ShoppingCart.AddItemByAdmin(
                            product, quantity, OptionItemValueCollection.Null, CartItemGiftDetails.Null, productCustomPrice, 0, isSubscriptionable );
                    }
                    else if (enterPrice != 0.0m)
                    {
                        StoreContext.ShoppingCart.AddItemByAdmin(
                            product, quantity, OptionItemValueCollection.Null, CartItemGiftDetails.Null, 0, enterPrice, isSubscriptionable );
                    }
                    else
                        StoreContext.ShoppingCart.AddItem( product, quantity, isSubscriptionable );

                }
            }
            else
            {
                OptionItemValueCollection inputLists = selectedOptions.GetOptionItemsByGroupType( OptionGroup.OptionGroupType.InputList );
                if (inputLists.Count == 0)
                {
                    OptionItemValueCollection optionsWithStock = selectedOptions.GetUseStockOptionItems();
                    if (VerifyStockOption(
                        product.ProductID, optionsWithStock.ConvertToIDArray(), quantity, out currentStock, storeID ))
                    {
                        //StoreContext.ShoppingCart.AddItem(
                        //    product, quantity, selectedOptions, productCustomPrice );
                        if (product.IsGiftCertificate)
                        {
                            StoreContext.ShoppingCart.AddItem( product, quantity, selectedOptions, giftDetails );
                        }
                        else
                        {
                            StoreContext.ShoppingCart.AddItemByAdmin(
                                product, quantity, selectedOptions, CartItemGiftDetails.Null, productCustomPrice, enterPrice, false );
                        }
                    }
                    else
                        result = false;
                }
                else
                {
                    OptionItemValueCollection optionsWithoutInputList =
                        selectedOptions.GetOptionItemsNotInGroupType( OptionGroup.OptionGroupType.InputList );

                    result = VerifyStockOptionWithInputList( product.ProductID, inputLists,
                        optionsWithoutInputList, out errorCurrentStock, out currentStock );

                    if (result)
                    {
                        // add product is Loop.
                        foreach (OptionItemValue optionInput in inputLists)
                        {
                            OptionItemValueCollection options = optionsWithoutInputList.Clone();
                            options.Add( optionInput );

                            //StoreContext.ShoppingCart.AddItem(
                            //    product, int.Parse( optionInput.Details ), options, productCustomPrice );

                            StoreContext.ShoppingCart.AddItemByAdmin(
                                product, int.Parse( optionInput.Details ), options, CartItemGiftDetails.Null, productCustomPrice, enterPrice, false );
                        }
                    }
                }
            }

            return result;
        }


        protected bool AddToCart(
            Product product,
            OptionItemValueCollection selectedOptions,
            ProductKitItemValueCollection valueCollection,
            int quantity,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice,
            out string errorCurrentStock,
            out int currentStock )
        {
            errorCurrentStock = String.Empty;
            bool result = true;

            if (product.IsProductKit)
            {
                foreach (ProductKitItemValue item in valueCollection)
                {
                    if (IsOutOfStock( item.ProductID, quantity * item.Quantity, out currentStock, out errorCurrentStock ))
                    {
                        return false;
                    }
                }
            }

            if (selectedOptions == null || selectedOptions.Count == 0)
            {
                if (IsOutOfStock( product.ProductID, quantity, out currentStock ))
                {
                    result = false;
                }
                else
                {
                    if (product.IsProductKit)
                    {
                        if (product.IsCustomPrice)
                        {
                            StoreContext.ShoppingCart.AddProductKitItem( product, quantity, OptionItemValueCollection.Null, valueCollection, productCustomPrice );
                        }
                        else
                        {
                            StoreContext.ShoppingCart.AddProductKitItem( product, quantity, OptionItemValueCollection.Null, valueCollection );
                        }
                    }
                    else
                    {
                        ProductSubscription productSubscription = new ProductSubscription( product.ProductID );

                        if (product.IsGiftCertificate)
                        {
                            StoreContext.ShoppingCart.AddItem(
                                product, quantity, giftDetails, productSubscription.IsSubscriptionProduct() );
                        }
                        else if (product.IsCustomPrice)
                        {
                            StoreContext.ShoppingCart.AddItem( product, quantity, productCustomPrice, productSubscription.IsSubscriptionProduct() );
                        }
                        else
                        {
                            StoreContext.ShoppingCart.AddItem( product, quantity, productSubscription.IsSubscriptionProduct() );
                        }
                    }
                }
            }
            else
            {
                OptionItemValueCollection inputLists = selectedOptions.GetOptionItemsByGroupType( OptionGroup.OptionGroupType.InputList );
                if (inputLists.Count == 0)
                {
                    OptionItemValueCollection optionsWithStock = selectedOptions.GetUseStockOptionItems();
                    if (VerifyStockOption(
                        product.ProductID, optionsWithStock.ConvertToIDArray(), quantity, out currentStock, new StoreRetriever().GetCurrentStoreID() ))
                    {
                        if (product.IsProductKit)
                        {
                            StoreContext.ShoppingCart.AddProductKitItem( product, quantity, selectedOptions, valueCollection, productCustomPrice );
                        }
                        else
                        {
                            StoreContext.ShoppingCart.AddItem(
                                product, quantity, selectedOptions, productCustomPrice );
                        }
                    }
                    else
                        result = false;
                }
                else
                {
                    OptionItemValueCollection optionsWithoutInputList =
                        selectedOptions.GetOptionItemsNotInGroupType( OptionGroup.OptionGroupType.InputList );

                    result = VerifyStockOptionWithInputList( product.ProductID, inputLists,
                        optionsWithoutInputList, out errorCurrentStock, out currentStock );

                    if (result)
                    {
                        // add product is Loop.
                        foreach (OptionItemValue optionInput in inputLists)
                        {
                            OptionItemValueCollection options = optionsWithoutInputList.Clone();
                            options.Add( optionInput );

                            if (product.IsProductKit)
                            {
                                StoreContext.ShoppingCart.AddProductKitItem( product, int.Parse( optionInput.Details ), options, valueCollection, productCustomPrice );
                            }
                            else
                            {
                                StoreContext.ShoppingCart.AddItem(
                                    product, int.Parse( optionInput.Details ), options, productCustomPrice );
                            }
                        }
                    }
                }
            }

            return result;
        }

        protected void uxTellFriendButton_Command( object sender, CommandEventArgs e )
        {
            string productID = e.CommandArgument.ToString();
            Response.Redirect( productID );
        }

        protected bool IsCallForPrice( object isCallForPrice )
        {
            if (ConvertUtilities.ToBoolean( isCallForPrice ))
                return true;
            else
                return false;
        }

        protected bool IsFixedPrice( object isFixedPrice )
        {
            return IsFixedPrice( isFixedPrice, false );
        }

        protected bool IsFixedPrice( object isFixedPrice, object isCustomPrice )
        {
            return IsFixedPrice( isFixedPrice, isCustomPrice, false );
        }

        protected bool IsFixedPrice( object isFixedPrice, object isCustomPrice, object isCallForPrice )
        {
            if (!IsAuthorizedToViewPrice())
                return false;

            if (ConvertUtilities.ToBoolean( isCallForPrice ))
                return false;

            if (ConvertUtilities.ToBoolean( isCustomPrice ))
                return false;

            if (ConvertUtilities.ToBoolean( isFixedPrice ))
                return true;
            else
                return false;
        }

        protected bool IsRetailPriceEnabled( object isFixedPrice, decimal retailPrice )
        {
            return IsRetailPriceEnabled( isFixedPrice, false, retailPrice );
        }

        protected bool IsRetailPriceEnabled( object isFixedPrice, object isCustomPrice, decimal retailPrice )
        {
            return IsRetailPriceEnabled( isFixedPrice, isCustomPrice, retailPrice, false );

        }

        protected bool IsRetailPriceEnabled( object isFixedPrice, object isCustomPrice, decimal retailPrice, object isCallForPrice )
        {
            if (!IsAuthorizedToViewPrice())
                return false;

            if (ConvertUtilities.ToBoolean( isCallForPrice ))
                return false;

            if (ConvertUtilities.ToBoolean( isCustomPrice ))
                return false;

            if (CatalogUtilities.IsRetailMode &&
                ConvertUtilities.ToBoolean( isFixedPrice ) &&
                ConvertUtilities.ToDecimal( retailPrice ) > 0
                )
                return true;
            else
                return false;
        }

        protected decimal GetRetailPrice( object productID )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( StoreContext.Culture, (string) productID, StoreContext.CurrentStore.StoreID );
            ProductPrice productPrice = product.GetProductPrice( StoreContext.CurrentStore.StoreID );

            return productPrice.RetailPrice;
        }

        protected string GetDiscountPercent( object productID )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( StoreContext.Culture, (string) productID, StoreContext.CurrentStore.StoreID );
            ProductPrice productPrice = product.GetProductPrice( StoreContext.CurrentStore.StoreID );

            if (productPrice.Price > 0 && productPrice.RetailPrice > 0)
            {
                decimal discount = (decimal) (((productPrice.RetailPrice - productPrice.Price) / productPrice.RetailPrice) * 100);
                if (discount < 1)
                    return "0%";
                else
                    return ((int) Math.Ceiling( discount )).ToString() + "%";
            }
            else
                return "0%";
        }

        protected bool IsDiscount( object productID )
        {
            if (GetDiscountPercent( productID ) != "0%")
                return true;
            else
                return false;

        }

        protected bool CheckEqualPrice( object productID )
        {
            Product product = DataAccessContext.ProductRepository.GetOne(
                      StoreContext.Culture,
                      productID.ToString(), StoreContext.CurrentStore.StoreID );

            ProductPrice productPrice = product.GetProductPrice( StoreContext.CurrentStore.StoreID );

            if (productPrice.RetailPrice.Equals( productPrice.Price ))
                return false;
            else
                return true;
        }

        protected string RemainingStock( object stock )
        {
            int amount = Convert.ToInt32( stock ) - DataAccessContext.Configurations.GetIntValue( "OutOfStockValue" );
            if (amount > 0)
            {
                return amount.ToString();
            }
            else
            {
                amount = 0;
                return amount.ToString();
            }
        }

        protected bool ShowRemainingQuantity( object useInventory )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "UseStockControl" ) &&
                DataAccessContext.Configurations.GetBoolValue( "ShowQuantity" ) &&
                ConvertUtilities.ToBoolean( useInventory ))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        protected bool IsTaxInclude()
        {
            return DataAccessContext.Configurations.GetBoolValue( "TaxIncludedInPrice" ) &&
                !String.IsNullOrEmpty( ConvertUtilities.ToString( Eval( "TaxClassID" ) ) ) &&
                ConvertUtilities.ToString( Eval( "TaxClassID" ) ) != "0";
        }

        protected bool IsDynamicProductKitPrice( Control bindingContainer )
        {
            Product currentProduct = GetProduct( bindingContainer );
            return currentProduct.IsDynamicProductKitPrice;
        }

        protected bool IsVisibilityRecurringPeriod()
        {
            return (ConvertUtilities.ToBoolean( Eval( "IsRecurring" ) )
                && !CatalogUtilities.IsOutOfStock( Eval( "SumStock" ), Eval( "UseInventory" ) )
                && CheckModeAndPermission());
        }

        protected bool CheckModeAndPermission()
        {
            return !CatalogUtilities.IsCatalogMode();
        }

        protected bool IsVisibilityTrialPeriodMore()
        {
            return IsVisibilityRecurringPeriod() &&
                (ConvertUtilities.ToInt32( Eval( "ProductRecurring.RecurringNumberOfTrialCycles" ) ) > 1) &&
                (ConvertUtilities.ToDecimal( Eval( "ProductRecurring.RecurringTrialAmount" ) ) > 0m);
        }

        protected bool IsVisibilityTrialPeriod()
        {
            return IsVisibilityRecurringPeriod() &&
                (ConvertUtilities.ToInt32( Eval( "ProductRecurring.RecurringNumberOfTrialCycles" ) ) == 1) &&
                (ConvertUtilities.ToDecimal( Eval( "ProductRecurring.RecurringTrialAmount" ) ) > 0m);
        }

        protected bool IsVisibilityFreeTrialPeriod()
        {
            return IsVisibilityRecurringPeriod() &&
                (ConvertUtilities.ToInt32( Eval( "ProductRecurring.RecurringNumberOfTrialCycles" ) ) > 1) &&
                (ConvertUtilities.ToDecimal( Eval( "ProductRecurring.RecurringTrialAmount" ) ) == 0m);
        }

        protected bool IsVisibilityFreeTrialPeriodMore()
        {
            return IsVisibilityRecurringPeriod() &&
                (ConvertUtilities.ToInt32( Eval( "ProductRecurring.RecurringNumberOfTrialCycles" ) ) == 1) &&
                (ConvertUtilities.ToDecimal( Eval( "ProductRecurring.RecurringTrialAmount" ) ) == 0m);
        }

        protected string ShowRecurringCycles()
        {
            if (ConvertUtilities.ToBoolean( Eval( "IsRecurring" ) ))
                return String.Format(
                    "{0} {1}s",
                    ConvertUtilities.ToInt32( Eval( "ProductRecurring.RecurringInterval" ) ),
                    Eval( "ProductRecurring.RecurringIntervalUnit" ).ToString().ToLower() );
            else
                return String.Empty;
        }

        protected string GetFormattedPriceFromContainer( Control bindingContainer )
        {
            Product currentProduct = GetProduct( bindingContainer );

            decimal price = currentProduct.GetDisplayedPrice( StoreContext.WholesaleStatus );

            return StoreContext.Currency.FormatPrice( price );
        }

        protected bool IsAuthorizedToViewPrice()
        {
            return IsAuthorizedToViewPrice( false );
        }

        protected bool IsAuthorizedToViewPrice( object isCallForPrice )
        {
            bool show = true;

            if (ConvertUtilities.ToBoolean( isCallForPrice ))
                return false;

            if (DataAccessContext.Configurations.GetBoolValue( "PriceRequireLogin" ) && !Page.User.Identity.IsAuthenticated)
            {
                show = false;
            }

            return show;
        }

        protected bool IsRmaEmable()
        {
            if (DataAccessContext.Configurations.GetBoolValue( "EnableRMA" ))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        protected string RmaMessage( object returnTime )
        {
            int day = Convert.ToInt32( returnTime );

            if (day == 0)
            {
                return GetLanguageText( "NoReturn" );
            }
            else
            {
                return GetLanguageText( "Return" ) + " " + day + " " + GetLanguageText( "ReturnUnit" );
            }
        }

        protected string ImageAlternateText( Control container, string productImageID )
        {
            Product currentProduct = GetProduct( container );
            if (currentProduct == null) return string.Empty;
            foreach (ProductImage image in currentProduct.ProductImages)
            {
                if (image.ProductImageID == productImageID)
                    return image.Locales[StoreContext.Culture].AltTag;
            }
            return string.Empty;
        }

        protected string ImageTitleText( Control container, string productImageID )
        {
            Product currentProduct = GetProduct( container );
            if (currentProduct == null) return string.Empty;
            foreach (ProductImage image in currentProduct.ProductImages)
            {
                if (image.ProductImageID == productImageID)
                    return image.Locales[StoreContext.Culture].TitleTag;
            }
            return string.Empty;
        }

        protected string ImageAlternateText( Control bindingContainer )
        {
            Product currentProduct = GetProduct( bindingContainer );
            if (currentProduct == null) return string.Empty;
            return currentProduct.GetPrimaryProductImage().Locales[StoreContext.Culture].AltTag;
        }

        protected string ImageTitleText( Control bindingContainer )
        {
            Product currentProduct = GetProduct( bindingContainer );
            if (currentProduct == null) return string.Empty;
            return currentProduct.GetPrimaryProductImage().Locales[StoreContext.Culture].TitleTag;
        }


        #endregion


        #region Contructors

        #endregion
    }

}
